/*
 * Copyright (c) 2004 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *   IBM - Initial API and implementation
 *
 * $Id: IntroductionToUML2Profiles.java,v 1.6 2004/10/29 14:34:33 khussey Exp $
 */
package com.ibm.uml2.articles;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.uml2.Extension;
import org.eclipse.uml2.Model;
import org.eclipse.uml2.NamedElement;
import org.eclipse.uml2.PrimitiveType;
import org.eclipse.uml2.Profile;
import org.eclipse.uml2.Property;
import org.eclipse.uml2.Stereotype;
import org.eclipse.uml2.UML2Factory;
import org.eclipse.uml2.UML2Package;
import org.eclipse.uml2.VisibilityKind;
import org.eclipse.uml2.util.UML2Resource;

/**
 * Source code for the "Introduction to UML2 Profiles" article.
 */
public class IntroductionToUML2Profiles
	extends UML2Article {

	protected static Profile createProfile(String name) {
		Profile profile = UML2Factory.eINSTANCE.createProfile();
		profile.setName(name);

		out("Profile '" + profile.getQualifiedName() + "' created.");

		return profile;
	}

	protected static PrimitiveType importPrimitiveType(
			org.eclipse.uml2.Package package_, String name) {
		Model uml2Library = (Model) load(URI
			.createURI(UML2Resource.UML2_PRIMITIVE_TYPES_LIBRARY_URI));

		PrimitiveType primitiveType = (PrimitiveType) uml2Library
			.getOwnedType(name);

		package_.importElement(VisibilityKind.PUBLIC_LITERAL, primitiveType);

		out("Primitive type '" + primitiveType.getQualifiedName()
			+ "' imported.");

		return primitiveType;
	}

	protected static Stereotype createStereotype(Profile profile, String name,
			boolean isAbstract) {
		Stereotype stereotype = (Stereotype) profile
			.createOwnedMember(UML2Package.eINSTANCE.getStereotype());
		stereotype.setName(name);
		stereotype.setIsAbstract(isAbstract);

		out("Stereotype '" + stereotype.getQualifiedName() + "' created.");

		return stereotype;
	}

	protected static org.eclipse.uml2.Class referenceMetaclass(Profile profile,
			String name) {
		Model uml2Metamodel = (Model) load(URI
			.createURI(UML2Resource.UML2_METAMODEL_URI));

		org.eclipse.uml2.Class metaclass = (org.eclipse.uml2.Class) uml2Metamodel
			.getOwnedType(name);

		profile.referenceMetaclass(metaclass);

		out("Metaclass '" + metaclass.getQualifiedName() + "' referenced.");

		return metaclass;
	}

	protected static Extension createExtension(EClass metaclass,
			Stereotype stereotype, boolean required) {
		Extension extension = stereotype.createExtension(metaclass, required);

		out((required
			? "Required extension '"
			: "Extension '") + extension.getQualifiedName() + "' created.");

		return extension;
	}

	protected static void defineProfile(Profile profile) {
		profile.define();

		out("Profile '" + profile.getQualifiedName() + "' defined.");
	}

	protected static void applyProfile(org.eclipse.uml2.Package package_,
			Profile profile) {
		package_.apply(profile);

		out("Profile '" + profile.getQualifiedName() + "' applied to package '"
			+ package_.getQualifiedName() + "'.");
	}

	protected static void applyStereotype(NamedElement namedElement,
			Stereotype stereotype) {
		namedElement.apply(stereotype);

		out("Stereotype '" + stereotype.getQualifiedName()
			+ "' applied to element '" + namedElement.getQualifiedName() + "'.");
	}

	protected static Object getStereotypePropertyValue(
			NamedElement namedElement, Stereotype stereotype, Property property) {
		Object value = namedElement.getValue(stereotype, property.getName());

		out("Value of stereotype property '" + property.getQualifiedName()
			+ "' on element '" + namedElement.getQualifiedName() + "' is "
			+ String.valueOf(value) + ".");

		return value;
	}

	protected static void setStereotypePropertyValue(NamedElement namedElement,
			Stereotype stereotype, Property property, Object value) {
		namedElement.setValue(stereotype, property.getName(), value);

		out("Value of stereotype property '" + property.getQualifiedName()
			+ "' on element '" + namedElement.getQualifiedName() + "' set to "
			+ String.valueOf(value) + ".");
	}

	public static void main(String[] args) {

		if (2 != args.length) {
			err("Usage: java IntroductionToUML2Profiles <URI> <URI>");
			System.exit(1);
		}

		registerResourceFactories();

		registerPathmaps(URI.createURI(args[1]));

		out("Creating profile...");

		Profile ecoreProfile = createProfile("ecore");

		out("Importing primitive types...");

		PrimitiveType booleanPrimitiveType = importPrimitiveType(ecoreProfile,
			"Boolean");
		PrimitiveType stringPrimitiveType = importPrimitiveType(ecoreProfile,
			"String");

		out("Creating stereotypes...");

		Stereotype eStructuralFeatureStereotype = createStereotype(
			ecoreProfile, "EStructuralFeature", true);

		Stereotype eAttributeStereotype = createStereotype(ecoreProfile,
			"EAttribute", false);

		Stereotype eReferenceStereotype = createStereotype(ecoreProfile,
			"EReference", false);

		out("Creating stereotype generalizations...");

		GettingStartedWithUML2.createGeneralization(eAttributeStereotype,
			eStructuralFeatureStereotype);

		GettingStartedWithUML2.createGeneralization(eReferenceStereotype,
			eStructuralFeatureStereotype);

		out("Creating stereotype properties...");

		Property isTransientProperty = GettingStartedWithUML2.createAttribute(
			eStructuralFeatureStereotype, "isTransient", booleanPrimitiveType,
			0, 1);
		Property isUnsettableProperty = GettingStartedWithUML2.createAttribute(
			eStructuralFeatureStereotype, "isUnsettable", booleanPrimitiveType,
			0, 1);
		Property isVolatileProperty = GettingStartedWithUML2.createAttribute(
			eStructuralFeatureStereotype, "isVolatile", booleanPrimitiveType,
			0, 1);

		Property attributeNameProperty = GettingStartedWithUML2
			.createAttribute(eAttributeStereotype, "attributeName",
				stringPrimitiveType, 0, 1);
		Property isIDProperty = GettingStartedWithUML2.createAttribute(
			eAttributeStereotype, "isID", booleanPrimitiveType, 0, 1);

		Property referenceNameProperty = GettingStartedWithUML2
			.createAttribute(eReferenceStereotype, "referenceName",
				stringPrimitiveType, 0, 1);
		Property isResolveProxiesProperty = GettingStartedWithUML2
			.createAttribute(eReferenceStereotype, "isResolveProxies",
				booleanPrimitiveType, 0, 1);

		isResolveProxiesProperty.setBooleanDefault(true);

		out("Referencing metaclasses...");

		referenceMetaclass(ecoreProfile, UML2Package.eINSTANCE.getProperty()
			.getName());

		out("Creating extensions...");

		createExtension(UML2Package.eINSTANCE.getProperty(),
			eAttributeStereotype, false);
		createExtension(UML2Package.eINSTANCE.getProperty(),
			eReferenceStereotype, false);

		out("Defining profile...");

		defineProfile(ecoreProfile);

		out("Saving profile...");

		save(ecoreProfile, URI.createURI(args[0]).appendSegment("Ecore")
			.appendFileExtension(UML2Resource.PROFILE_FILE_EXTENSION));

		out("Loading model...");

		Model epo2Model = (Model) load(URI.createURI(args[0]).appendSegment(
			"ExtendedPO2").appendFileExtension(UML2Resource.FILE_EXTENSION));

		out("Applying profile...");

		applyProfile(epo2Model, ecoreProfile);

		out("Applying stereotypes...");

		org.eclipse.uml2.Class supplierClass = (org.eclipse.uml2.Class) epo2Model
			.getOwnedType("Supplier");

		Property pendingOrdersProperty = supplierClass
			.getOwnedAttribute("pendingOrders");
		applyStereotype(pendingOrdersProperty, eReferenceStereotype);

		Property shippedOrdersProperty = supplierClass
			.getOwnedAttribute("shippedOrders");
		applyStereotype(shippedOrdersProperty, eReferenceStereotype);

		org.eclipse.uml2.Class purchaseOrderClass = (org.eclipse.uml2.Class) epo2Model
			.getOwnedType("PurchaseOrder");

		Property totalAmountProperty = purchaseOrderClass
			.getOwnedAttribute("totalAmount");
		applyStereotype(totalAmountProperty, eAttributeStereotype);

		Property previousOrderProperty = purchaseOrderClass
			.getOwnedAttribute("previousOrder");
		applyStereotype(previousOrderProperty, eReferenceStereotype);

		Property customerProperty = purchaseOrderClass
			.getOwnedAttribute("customer");
		applyStereotype(customerProperty, eReferenceStereotype);

		org.eclipse.uml2.Class customerClass = (org.eclipse.uml2.Class) epo2Model
			.getOwnedType("Customer");

		Property ordersProperty = customerClass.getOwnedAttribute("orders");
		applyStereotype(ordersProperty, eReferenceStereotype);

		out("Getting stereotype property values...");

		getStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isVolatileProperty);
		getStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isTransientProperty);
		getStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isVolatileProperty);
		getStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isTransientProperty);
		getStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(totalAmountProperty, eAttributeStereotype,
			isVolatileProperty);
		getStereotypePropertyValue(totalAmountProperty, eAttributeStereotype,
			isTransientProperty);

		getStereotypePropertyValue(previousOrderProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(customerProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(ordersProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		out("Setting stereotype property values...");

		setStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isVolatileProperty, Boolean.TRUE);
		setStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isTransientProperty, Boolean.TRUE);
		setStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isResolveProxiesProperty, Boolean.FALSE);

		setStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isVolatileProperty, Boolean.TRUE);
		setStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isTransientProperty, Boolean.TRUE);
		setStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isResolveProxiesProperty, Boolean.FALSE);

		setStereotypePropertyValue(totalAmountProperty, eAttributeStereotype,
			isVolatileProperty, Boolean.TRUE);
		setStereotypePropertyValue(totalAmountProperty, eAttributeStereotype,
			isTransientProperty, Boolean.TRUE);

		setStereotypePropertyValue(previousOrderProperty, eReferenceStereotype,
			isResolveProxiesProperty, Boolean.FALSE);

		setStereotypePropertyValue(customerProperty, eReferenceStereotype,
			isResolveProxiesProperty, Boolean.FALSE);

		setStereotypePropertyValue(ordersProperty, eReferenceStereotype,
			isResolveProxiesProperty, Boolean.FALSE);

		out("Getting stereotype property values...");

		getStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isVolatileProperty);
		getStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isTransientProperty);
		getStereotypePropertyValue(pendingOrdersProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isVolatileProperty);
		getStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isTransientProperty);
		getStereotypePropertyValue(shippedOrdersProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(totalAmountProperty, eAttributeStereotype,
			isVolatileProperty);
		getStereotypePropertyValue(totalAmountProperty, eAttributeStereotype,
			isTransientProperty);

		getStereotypePropertyValue(previousOrderProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(customerProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		getStereotypePropertyValue(ordersProperty, eReferenceStereotype,
			isResolveProxiesProperty);

		out("Saving model...");

		save(epo2Model, URI.createURI(args[0]).appendSegment(
			"ExtendedPO2_Ecore").appendFileExtension(
			UML2Resource.FILE_EXTENSION));
	}
}